@@ -1,8 +1,39 @@
+import sys
 import os
 import inspect
 import plac
 import plac_ini
 
+# Define exec_ for python 2 and 3. Taken from six
+if sys.version > '3':
+    exec_ = exec
+
+
+    def reraise(tp, value, tb=None):
+        if value is None:
+            value = tp()
+        if value.__traceback__ is not tb:
+            raise value.with_traceback(tb)
+        raise value
+
+else:
+    def exec_(_code_, _globs_=None, _locs_=None):
+        """Execute code in a namespace."""
+        if _globs_ is None:
+            frame = sys._getframe(1)
+            _globs_ = frame.f_globals
+            if _locs_ is None:
+                _locs_ = frame.f_locals
+            del frame
+        elif _locs_ is None:
+            _locs_ = _globs_
+        exec("""exec _code_ in _globs_, _locs_""")
+
+
+    exec_("""def reraise(tp, value, tb=None):
+    raise tp, value, tb
+""")
+
 
 # Based on a decorator that modified the call signature for a function
 # http://www.pythoneye.com/184_18642398/
@@ -29,7 +60,8 @@ class configure(object):
     """
     def __init__(self, **ann):
         self.ann = ann
-        self.config = {}
+        self.wrapped = None
+        self.__config = {}
 
     def __call__(self, tgt_func):
         tgt_argspec = inspect.getargspec(tgt_func)
@@ -82,8 +114,8 @@ class configure(object):
                 '    return %(tgt_func)s(%(signature)s)\n' %
                 {'signature':signature, 'newsignature':newsignature, 'tgt_func':'tgt_func'}
             )
-        evaldict = {'tgt_func' : tgt_func, 'plac' : plac, 'config' : self.config}
-        exec new_func in evaldict
+        evaldict = {'tgt_func' : tgt_func, 'plac' : plac, 'config' : self.__config}
+        exec_(new_func, evaldict)
         wrapped = evaldict['_wrapper_']
 
         # Update the wrapper with all of the information from the wrapped function
@@ -93,19 +125,57 @@ class configure(object):
         wrapped.__module__ = tgt_func.__module__
         wrapped.__dict__ = tgt_func.__dict__
 
-        # Add the complete annotations to the wrapper functoin, and also add the getconfig method
+        # Add the complete annotations to the wrapper function, and also add the getconfig method
         # so that the new arguments can be retrieved inside the wrapped function.
         # This must come after the __dict__ assignment
         wrapped.__annotations__ = self.ann
         wrapped.getconfig = self.getconfig
+        self.wrapped = wrapped
         return wrapped
 
-    def getconfig(self):
+    def getconfig(self, section=None):
         """
         This method provides a way for decorated functions to get the
-        four new configuration parameters.
+        four new configuration parameters *after* it has been called.
+
+        If no section is specified, then the fully resolved zdesk
+        config will be returned. That is defaults, zdesk ini section,
+        command line options.
+
+        If a section is specified, then the same rules apply, but also
+        any missing items are filled in by the zdesk section. So the
+        resolution is defaults, zdesk ini section, specified section,
+        command line options.
         """
-        return self.config
+        if not section:
+            return self.__config.copy()
+
+        cmd_line = {}
+        for k in self.__config:
+            v = self.wrapped.plac_cfg.get(k, 'PLAC__NOT_FOUND')
+            if v != self.__config[k]:
+                # This config item is different when fully resolved
+                # compared to the ini value. It was specified on the
+                # command line.
+                cmd_line[k] = self.__config[k]
+
+        # Get the email, password, url, and token config from the indicated
+        # section, falling back to the zdesk config for convenience
+        cfg = {
+            "zdesk_email": self.wrapped.plac_cfg.get(section + '_email',
+                self.__config['zdesk_email']),
+            "zdesk_password": self.wrapped.plac_cfg.get(section + '_password',
+                self.__config['zdesk_password']),
+            "zdesk_url": self.wrapped.plac_cfg.get(section + '_url',
+                self.__config['zdesk_url']),
+            "zdesk_token": self.wrapped.plac_cfg.get(section + '_token',
+                self.__config['zdesk_token']),
+        }
+
+        # The command line trumps all
+        cfg.update(cmd_line)
+
+        return cfg
 
 def call(obj, config=os.path.join(os.path.expanduser('~'), '.zdeskcfg'), section=None):
     plac_ini.call(obj, config=config, default_section=section)
